"use strict";

Object.defineProperty(exports, "__esModule", {
    value: !0
}), exports.Plot = void 0;

const vutils_1 = require("@visactor/vutils"), util_1 = require("./util"), enums_1 = require("./enums"), vgrammar_core_1 = require("@visactor/vgrammar-core");

class Plot {
    static useMarks(marks) {
        marks.forEach((mark => {
            vgrammar_core_1.Factory.registerPlotMarks(mark.type, mark);
        }));
    }
    constructor(option) {
        this.view = new vgrammar_core_1.View(option), this._semanticMarks = [], this._logger = vutils_1.Logger.getInstance();
    }
    theme(theme) {
        return this._theme = theme, this.view.parseSpec({
            theme: theme
        }), this;
    }
    _mergeScales(scales, prevScales) {
        return scales.reduce(((res, scale) => {
            if (scale.id) {
                const prevIndex = res.findIndex((prev => prev.id === scale.id));
                if (prevIndex >= 0) {
                    const prevScale = res[prevIndex];
                    if (scale.domain.data && scale.domain.field) if (prevScale.domain.data && prevScale.domain.field) scale.domain.data === prevScale.domain.data && scale.domain.field !== prevScale.domain.field ? prevScale.domain.field = [].concat(prevScale.domain.field).concat(scale.domain.field) : scale.domain.data !== prevScale.domain.data && (prevScale.domain = {
                        datas: [ {
                            data: prevScale.domain.data,
                            field: prevScale.domain.field
                        }, {
                            data: scale.domain.data,
                            field: scale.domain.field
                        } ],
                        sort: prevScale.domain.sort
                    }), (0, vutils_1.isNil)(scale.domain.sort) || (prevScale.domain.sort = scale.domain.sort); else if (prevScale.domain.datas) {
                        const prevData = prevScale.domain.datas.find((entry => entry.data === scale.domain.data));
                        prevData && scale.domain.field !== prevData.field ? prevData.field = [].concat(prevData.field).concat(scale.domain.field) : prevData || prevScale.domain.datas.push({
                            data: scale.domain.data,
                            field: scale.domain.field
                        }), (0, vutils_1.isNil)(scale.domain.sort) || (prevScale.domain.sort = scale.domain.sort);
                    }
                    scale.userScale && (res[prevIndex] = (0, vutils_1.merge)(prevScale, scale.userScale));
                } else scale.userScale = null, res.push(scale);
            }
            return res;
        }), prevScales);
    }
    parseViewSpec() {
        const spec = {
            theme: this._theme,
            data: [],
            interactions: [],
            marks: [],
            scales: [],
            coordinates: [],
            signals: [],
            projections: [],
            events: []
        };
        return this._semanticMarks.forEach((mark => {
            this._coordinate && mark.coordinate(this._coordinate);
            const {data: data, marks: marks, scales: scales, coordinates: coordinates, signals: signals, projections: projections, events: events, interactions: interactions} = mark.toViewSpec();
            data && data.length && (spec.data = (0, util_1.mergeGrammarSpecs)(data, spec.data)), 
            marks && marks.length && (spec.marks = spec.marks.concat(marks)), scales && scales.length && (spec.scales = this._mergeScales(scales, spec.scales)), 
            coordinates && coordinates.length && (spec.coordinates = (0, util_1.mergeGrammarSpecs)(coordinates, spec.coordinates)), 
            signals && signals.length && (spec.signals = spec.signals.concat(signals)), projections && projections.length && (spec.projections = spec.projections.concat(projections)), 
            events && events.length && (spec.events = spec.events.concat(events)), interactions && interactions.length && (spec.interactions = spec.interactions.concat(interactions));
        })), spec.marks = [ {
            type: "group",
            id: "plotContainer",
            layout: {
                display: "relative",
                updateViewSignals: !0
            },
            dependency: [ vgrammar_core_1.SIGNAL_VIEW_BOX ],
            encode: {
                update: (datum, elment, params) => ({
                    x: params.viewBox.x1,
                    y: params.viewBox.y1,
                    width: params.viewBox.width(),
                    height: params.viewBox.height()
                })
            },
            marks: spec.marks
        } ], spec;
    }
    run(morphConfig) {
        return this.view && (this._hasInited ? this.view.updateSpec(this.parseViewSpec()) : this.view.parseSpec(this.parseViewSpec()), 
        this._hasInited = !0, this.view.run(morphConfig)), this;
    }
    release() {
        return this.view && this.view.release(), this;
    }
    updateSpec(spec) {
        return this.parseSpec(spec, !0), this;
    }
    parseSpec(spec, isUpdate) {
        var _a;
        spec.coordinate && this.coordinate(spec.coordinate.type, spec.coordinate), (null === (_a = null == spec ? void 0 : spec.marks) || void 0 === _a ? void 0 : _a.length) && spec.marks.forEach((mark => {
            const plotMark = vgrammar_core_1.Factory.createPlotMark(mark.type);
            plotMark.parseSpec(mark), this._semanticMarks.push(plotMark);
        }));
        const viewSpec = this.parseViewSpec();
        return viewSpec.width = spec.width, viewSpec.height = spec.height, viewSpec.background = spec.background, 
        viewSpec.padding = spec.padding, isUpdate ? this.view.updateSpec(viewSpec) : this.view.parseSpec(viewSpec), 
        this._hasInited = !0, this;
    }
    getImageBuffer() {
        var _a, _b;
        return null === (_b = null === (_a = this.view) || void 0 === _a ? void 0 : _a.getImageBuffer) || void 0 === _b ? void 0 : _b.call(_a);
    }
    on(type, handler) {
        return this.view && this.view.addEventListener(type, handler), this;
    }
    off(type, handler) {
        return this.view && this.view.removeEventListener(type, handler), this;
    }
    getCoordinateId(viewId = "0") {
        return `coordinate-${viewId}`;
    }
    coordinate(type, spec) {
        return this._coordinate = Object.assign({
            type: type,
            id: this.getCoordinateId()
        }, spec), this;
    }
    interval() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.interval);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.interval} before use it`), 
        mark;
    }
    line() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.line);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.line} before use it`), 
        mark;
    }
    area() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.area);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.area} before use it`), 
        mark;
    }
    cell() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.cell);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.cell} before use it`), 
        mark;
    }
    ruleX() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.ruleX);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${this.ruleX} before use it`), 
        mark;
    }
    ruleY() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.ruleY);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.ruleY} before use it`), 
        mark;
    }
    image() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.image);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.image} before use it`), 
        mark;
    }
    path() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.path);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.path} before use it`), 
        mark;
    }
    polygon() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.polygon);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.polygon} before use it`), 
        mark;
    }
    rectX() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.rectX);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.rectX} before use it`), 
        mark;
    }
    rectY() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.rectY);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.rectY} before use it`), 
        mark;
    }
    rect() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.rect);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.rect} before use it`), 
        mark;
    }
    rule() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.rule);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.rule} before use it`), 
        mark;
    }
    symbol() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.symbol);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.symbol} before use it`), 
        mark;
    }
    text() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.text);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.text} before use it`), 
        mark;
    }
    sankey() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.sankey);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.sankey} before use it`), 
        mark;
    }
    sunburst() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.sunburst);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.sunburst} before use it`), 
        mark;
    }
    tree() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.tree);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.tree} before use it`), 
        mark;
    }
    treemap() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.treemap);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.treemap} before use it`), 
        mark;
    }
    circlePacking() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.circlePacking);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.circlePacking} before use it`), 
        mark;
    }
    wordcloud() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.wordcloud);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.wordcloud} before use it`), 
        mark;
    }
    wordcloudShape() {
        const mark = vgrammar_core_1.Factory.createPlotMark(enums_1.PlotMakType.wordcloudShape);
        return mark ? this._semanticMarks.push(mark) : this._logger.error(`Please register ${enums_1.PlotMakType.wordcloudShape} before use it`), 
        mark;
    }
}

exports.Plot = Plot;
//# sourceMappingURL=plot.js.map